from urllib.parse import urlencode


class ImagingStudy:
    def __init__(self, imaging_study_id=''):
        self.imaging_study_id = imaging_study_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'ImagingStudy',
            'id': imaging_study_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'subject': {},
            'encounter': {},
            'started': '',
            'interpreter': [],
            'description': '',
        }

        if imaging_study_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_encounter(self, encounter: dict):
        self.payload_template['encounter'] = encounter

    def set_started(self, started: str):
        self.payload_template['started'] = started

    def set_interpreter(self, interpreter: list):
        self.payload_template['interpreter'] = interpreter

    def set_description(self, description: str):
        self.payload_template['description'] = description

    def create(self):
        return self.payload_template

    def build_imaging_study_id_query(self, imaging_study_id: str):
        return urlencode({'_id': imaging_study_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



import requests
from urllib.parse import urlencode


class ClientCredentials:
    def __init__(self, client_id: str, client_secret: str, req_url: str):
        self.client_id = client_id
        self.client_secret = client_secret
        self.grant_type = 'client_credentials'
        self.scope = 'openid'

        self.req_url = req_url

        self.headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Accept': 'application/json'
        }

        self.response = None

    def send(self):
        payload = f'client_id={self.client_id}&client_secret={self.client_secret}&grant_type={self.grant_type}&scope={self.scope}'

        response = requests.post(self.req_url, headers=self.headers, data=payload)
        self.response = response

        return response

    def retrieve_token(self):
        if self.response.status_code != 200:
            raise Exception(f'The response status code is {self.response.status_code}')

        return self.response.json().get('access_token')



from urllib.parse import urlencode


class DocumentReference:
    def __init__(self, document_reference_id=''):
        self.document_reference_id = document_reference_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'DocumentReference',
            'id': document_reference_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'type': {
                'coding': [],
                'text': '',
            },
            'subject': {},
            'date': '',
            'author': [],
            'custodian': {},
            'content': [],
        }

        if document_reference_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_type_coding(self, type_coding: list):
        self.payload_template['type']['coding'] = type_coding

    def set_type_text(self, type_text: str):
        self.payload_template['type']['text'] = type_text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_date(self, date: str):
        self.payload_template['date'] = date

    def set_author(self, author: list):
        self.payload_template['author'] = author

    def set_custodian(self, custodian: dict):
        self.payload_template['custodian'] = custodian

    def set_content(self, content: list):
        self.payload_template['content'] = content

    def create(self):
        return self.payload_template

    def build_document_reference_id_query(self, document_reference_id: str):
        return urlencode({'_id': document_reference_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Organization:
    def __init__(self, organization_id=''):
        self.organization_id = organization_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Organization',
            'id': organization_id,
            'meta': {
                'profile': [],
            },
            'identifier': [],
            'type': [{
                'coding': [],
            }],
            'name': '',
            'telecom': [],
        }

        if organization_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifiers(self, identifiers: list):
        self.payload_template['identifier'] = identifiers

    def set_type_coding(self, type_coding: list):
        self.payload_template['type'][0]['coding'] = type_coding

    def set_name(self, name: str):
        self.payload_template['name'] = name

    def set_telecom(self, telecom: list):
        self.payload_template['telecom'] = telecom

    def create(self):
        if len(self.payload_template['telecom']) == 0:
            del self.payload_template['telecom']

        return self.payload_template

    def build_organization_id_query(self, organization_id: str):
        return urlencode({'_id': organization_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class PractitionerRole:
    def __init__(self, practitioner_role_id=''):
        self.practitioner_role_id = practitioner_role_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'PractitionerRole',
            'id': practitioner_role_id,
            'meta': {
                'profile': [],
            },
            'identifier' : [],
            'active': None,
            'period': {},
            'practitioner': {},
            'code': [],
            'specialty': [{
                'coding': [],
            }],
            'location': [],
            'telecom': [],
            'availableTime': [],
            'notAvailable': [],
            'availabilityExceptions': '',
        }

        if practitioner_role_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifiers(self, identifiers: list):
        self.payload_template['identifier'] = identifiers

    def set_active(self, active: bool):
        self.payload_template['active'] = active

    def set_period(self, period: dict):
        self.payload_template['period'] = period

    def set_practitioner(self, practitioner: dict):
        self.payload_template['practitioner'] = practitioner

    def set_code(self, code: list):
        self.payload_template['code'] = code

    def set_specialty_coding(self, specialty_coding: list):
        self.payload_template['specialty'][0]['coding'] = specialty_coding

    def set_location(self, location: list):
        self.payload_template['location'] = location

    def set_telecom(self, telecom: list):
        self.payload_template['telecom'] = telecom

    def set_available_time(self, available_time: list):
        self.payload_template['availableTime'] = available_time

    def set_not_available(self, not_available: list):
        self.payload_template['notAvailable'] = not_available

    def set_availability_exceptions(self, availability_exception: str):
        self.payload_template['availabilityExceptions'] = availability_exception

    def create(self):
        if self.payload_template['active'] is None:
            del self.payload_template['active']
        if self.payload_template['availabilityExceptions'] == '':
            del self.payload_template['availabilityExceptions']
        if len(self.payload_template['availableTime']) == 0:
            del self.payload_template['availableTime']
        if len(self.payload_template['identifier']) == 0:
            del self.payload_template['identifier']
        if len(self.payload_template['location']) == 0:
            del self.payload_template['location']
        if len(self.payload_template['notAvailable']) == 0:
            del self.payload_template['notAvailable']
        if self.payload_template['period'] == {}:
            del self.payload_template['period']
        if self.payload_template['specialty'] == [{'coding': []}]:
            del self.payload_template['specialty']
        if len(self.payload_template['telecom']) == 0:
            del self.payload_template['telecom']

        return self.payload_template

    def build_practitioner_role_id_query(self, practitioner_role_id: str):
        return urlencode({'_id': practitioner_role_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Medication:
    def __init__(self, medication_id=''):
        self.medication_id = medication_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Medication',
            'id': medication_id,
            'meta': {
                'profile': [],
            },
            'code': {
                'coding': [],
                'text': '',
            },
            'form': {
                'coding': [],
                'text': '',
            },
        }

        if medication_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_code_coding(self, coding: list):
        self.payload_template['code']['coding'] = coding

    def set_code_text(self, code_text: str):
        self.payload_template['code']['text'] = code_text

    def set_form_coding(self, form_coding: list):
        self.payload_template['form']['coding'] = form_coding

    def set_form_text(self, form_text: str):
        self.payload_template['form']['text'] = form_text

    def create(self):
        return self.payload_template

    def build_encounter_id_query(self, encounter_id: str):
        return urlencode({'_id': encounter_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Claim:
    def __init__(self, claim_id=''):
        self.claim_id = claim_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Claim',
            'id': claim_id,
            'meta': {
                'profile': [],
            },
            'identifier': [{}],
            'status': '',
            'type': {
                'coding': [],
            },
            'use': 'claim',
            'patient': {},
            'created': '',
            'provider': {},
            'priority': {
                'coding': [],
            },
            'diagnosis': [],
            'insurance': [],
            'item': [],
            'total': {},
        }

        if claim_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_type_coding(self, type_coding: list):
        self.payload_template['type']['coding'] = type_coding

    def set_patient(self, patient: dict):
        self.payload_template['patient'] = patient

    def set_created(self, created: str):
        self.payload_template['created'] = created

    def set_provider(self, provider: dict):
        self.payload_template['provider'] = provider

    def set_priority_coding(self, coding: list):
        self.payload_template['priority']['coding'] = coding

    def set_diagnosis(self, diagnosis: list):
        self.payload_template['diagnosis'] = diagnosis

    def set_insurance(self, insurance: list):
        self.payload_template['insurance'] = insurance

    def set_item(self, item: list):
        self.payload_template['item'] = item

    def set_total(self, total: dict):
        self.payload_template['total'] = total

    def create(self):
        return self.payload_template

    def build_composition_id_query(self, composition_id: str):
        return urlencode({'_id': composition_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class MedicationStatement:
    def __init__(self, medication_statement_id=''):
        self.medication_statement_id = medication_statement_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'MedicationStatement',
            'id': medication_statement_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'category': {
                'coding': [],
                'text': '',
            },
            'medicationCodeableConcept': {
                'coding': [],
                'text': '',
            },
            'subject': {},
            'effectiveDateTime': '',
            'dateAsserted': '',
            'reasonCode': [{
                'coding': [],
                'text': '',
            }],
            'note': [],
            'dosage': [{
                'text': '',
                'timing': {
                    'repeat': {},
                },
                'route': {
                    'coding': [],
                    'text': '',
                },
            }],
        }

        if medication_statement_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_category_coding(self, category_coding: list):
        self.payload_template['category']['coding'] = category_coding

    def set_category_text(self, text: str):
        self.payload_template['category']['text'] = text

    def set_medication_codeable_concept_coding(self, concept_coding: list):
        self.payload_template['medicationCodeableConcept']['coding'] = concept_coding

    def set_medication_codeable_concept_text(self, text: str):
        self.payload_template['medicationCodeableConcept']['text'] = text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_effective_date_time(self, effective_date_time: str):
        self.payload_template['effectiveDateTime'] = effective_date_time

    def set_date_asserted(self, date_asserted: str):
        self.payload_template['dateAsserted'] = date_asserted

    def set_reason_code_coding(self, code_coding: list):
        self.payload_template['reasonCode'][0]['coding'] = code_coding

    def set_reason_code_text(self, code_text: str):
        self.payload_template['reasonCode'][0]['text'] = code_text

    def set_note(self, note: list):
        self.payload_template['note'] = note

    def set_dosage_text(self, text: str):
        self.payload_template['dosage'][0]['text'] = text

    def set_dosage_timing_repeat(self, timing_repeat: dict):
        self.payload_template['dosage'][0]['timing']['repeat'] = timing_repeat

    def set_dosage_route_coding(self, route_coding: list):
        self.payload_template['dosage'][0]['route']['coding'] = route_coding

    def set_dosage_route_text(self, route_text: str):
        self.payload_template['dosage'][0]['route']['text'] = route_text

    def create(self):
        return self.payload_template

    def build_medication_statement_id_query(self, medication_statement_id: str):
        return urlencode({'_id': medication_statement_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Media:
    def __init__(self, media_id=''):
        self.media_id = media_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Media',
            'id': media_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'type': {
                'coding': [],
            },
            'view': {
                'coding': [],
                'text': '',
            },
            'subject': {},
            'createdDateTime': '',
            'issued': '',
            'operator': {},
            'reasonCode': [{
                'coding': [],
            }],
            'bodySite': {
                'coding': [],
            },
            'deviceName': '',
            'height': None,
            'width': None,
            'content': {},
            'note': [],
        }

        if media_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_type_coding(self, type_coding: list):
        self.payload_template['type']['coding'] = type_coding

    def set_view_coding(self, view_coding: list):
        self.payload_template['view']['coding'] = view_coding

    def set_view_text(self, view_text: str):
        self.payload_template['view']['text'] = view_text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_created_datetime(self, created_datetime: str):
        self.payload_template['createdDateTime'] = created_datetime

    def set_issued(self, issued: str):
        self.payload_template['issued'] = issued

    def set_operator(self, operator: dict):
        self.payload_template['operator'] = operator

    def set_reason_code_coding(self, reason_code_coding: list):
        self.payload_template['reasonCode'][0]['coding'] = reason_code_coding

    def set_body_site_coding(self, body_site_coding: list):
        self.payload_template['bodySite']['coding'] = body_site_coding

    def set_device_name(self, device_name: str):
        self.payload_template['deviceName'] = device_name

    def set_height(self, height: int):
        self.payload_template['height'] = height

    def set_width(self, width: int):
        self.payload_template['width'] = width

    def set_content(self, content: dict):
        self.payload_template['content'] = content

    def set_note(self, note: list):
        self.payload_template['note'] = note

    def create(self):
        return self.payload_template

    def build_media_id_query(self, media_id: str):
        return urlencode({'_id': media_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Goal:
    def __init__(self, goal_id=''):
        self.goal_id = goal_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Goal',
            'id': goal_id,
            'meta': {
                'profile': [],
            },
            'identifier' : [],
            'lifecycleStatus': '',
            'category': [{
                'coding': [],
            }],
            'description': {
                'text': '',
            },
            'subject': {},
            'target': [{
                'measure': {
                    'coding': [],
                },
                'detailQuantity': {},
            }],
        }

        if goal_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifiers(self, identifiers: list):
        self.payload_template['identifier'] = identifiers

    def set_lifecycle_status(self, lifecycle_status: str):
        self.payload_template['lifecycleStatus'] = lifecycle_status

    def set_category_coding(self, coding: list):
        self.payload_template['category'][0]['coding'] = coding

    def set_description_text(self, text: str):
        self.payload_template['description']['text'] = text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_target_measure_coding(self, target_measure_coding: list):
        self.payload_template['target'][0]['measure']['coding'] = target_measure_coding

    def set_target_detail_quantity(self, target_detail_quantity: dict):
        self.payload_template['target'][0]['detailQuantity'] = target_detail_quantity

    def create(self):
        return self.payload_template

    def build_goal_id_query(self, goal_id: str):
        return urlencode({'_id': goal_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Practitioner:
    def __init__(self, practitioner_id=''):
        self.practitioner_id = practitioner_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Practitioner',
            'id': practitioner_id,
            'meta': {
                'profile': [],
            },
            'identifier' : [],
            'active': None,
            'name': [],
            'telecom': [],
            'address': [],
            'gender': '',
            'birthDate': '',
            'photo': [],
            'qualification': [],
        }

        if practitioner_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifiers(self, identifiers: list):
        self.payload_template['identifier'] = identifiers

    def set_active(self, active: bool):
        self.payload_template['active'] = active

    def set_name(self, name: list):
        self.payload_template['name'] = name

    def set_telecom(self, telecom: list):
        self.payload_template['telecom'] = telecom

    def set_address(self, address: list):
        self.payload_template['address'] = address

    def set_gender(self, gender: str):
        self.payload_template['gender'] = gender

    def set_birth_date(self, birth_date: str):
        self.payload_template['birthDate'] = birth_date

    def set_photo(self, photo: list):
        self.payload_template['photo'] = photo

    def set_qualification(self, qualification: list):
        self.payload_template['qualification'] = qualification

    def create(self):
        if self.payload_template['active'] is None:
            del self.payload_template['active']
        if len(self.payload_template['address']) == 0:
            del self.payload_template['address']
        if self.payload_template['birthDate'] == '':
            del self.payload_template['birthDate']
        if self.payload_template['gender'] == '':
            del self.payload_template['gender']
        if len(self.payload_template['photo']) == 0:
            del self.payload_template['photo']
        if len(self.payload_template['qualification']) == 0:
            del self.payload_template['qualification']
        if len(self.payload_template['telecom']) == 0:
            del self.payload_template['telecom']
        if len(self.payload_template['identifier']) == 0:
            del self.payload_template['identifier']

        return self.payload_template

    def build_practitioner_id_query(self, practitioner_id: str):
        return urlencode({'_id': practitioner_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Procedure:
    def __init__(self, procedure_id=''):
        self.procedure_id = procedure_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Procedure',
            'id': procedure_id,
            'meta': {
                'profile': [],
            },
            'partOf': [],
            'status': '',
            'category': {
                'coding': [],
            },
            'code': {
                'coding': [],
                'text': '',
            },
            'subject': {},
            'encounter': {},
            'performedDateTime': '',
            'asserter': {},
            'performer': [],
            'bodySite': [{
                'coding': [],
            }],
            'reasonReference': [],
            'report': [],
            'note': [],
            'usedCode': [],
        }

        if procedure_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_part_of(self, part_of: list):
        self.payload_template['partOf'] = part_of

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_category(self, category: dict):
        self.payload_template['category'] = category

    def set_code_coding(self, coding: list):
        self.payload_template['code']['coding'] = coding

    def set_code_text(self, text: str):
        self.payload_template['code']['text'] = text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_encounter(self, encounter: dict):
        self.payload_template['encounter'] = encounter

    def set_performed_date_time(self, performed_date_time: str):
        self.payload_template['performedDateTime'] = performed_date_time

    def set_asserter(self, asserter: dict):
        self.payload_template['asserter'] = asserter

    def set_performer(self, performer: list):
        self.payload_template['performer'] = performer

    def set_body_site_coding(self, body_site_coding: list):
        self.payload_template['bodySite'][0]['coding'] = body_site_coding

    def set_reason_reference(self, reason_reference: list):
        self.payload_template['reasonReference'] = reason_reference

    def set_report(self, report: list):
        self.payload_template['report'] = report

    def set_note(self, note: list):
        self.payload_template['note'] = note

    def set_used_code(self, used_code: list):
        self.payload_template['usedCode'] = used_code

    def create(self):
        if self.payload_template['partOf'] == []:
            del self.payload_template['partOf']
        if self.payload_template['category'] == {'coding': []}:
            del self.payload_template['category']
        if self.payload_template['encounter'] == {}:
            del self.payload_template['encounter']
        if self.payload_template['performedDateTime'] == '':
            del self.payload_template['performedDateTime']
        if self.payload_template['asserter'] == {}:
            del self.payload_template['asserter']
        if self.payload_template['performer'] == []:
            del self.payload_template['performer']
        if self.payload_template['bodySite'] == [{'coding': []}]:
            del self.payload_template['bodySite']
        if self.payload_template['report'] == []:
            del self.payload_template['report']
        if self.payload_template['note'] == []:
            del self.payload_template['note']
        if self.payload_template['usedCode'] == []:
            del self.payload_template['usedCode']
        if self.payload_template['reasonReference'] == []:
            del self.payload_template['reasonReference']
        if self.payload_template['code']['text'] == '':
            del self.payload_template['code']['text']

        return self.payload_template

    def build_procedure_role_id_query(self, procedure_id: str):
        return urlencode({'_id': procedure_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class CarePlan:
    def __init__(self, care_plan_id=''):
        self.care_plan_id = care_plan_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'CarePlan',
            'id': care_plan_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'intent': 'plan',
            'category': [{
                'coding': [],
            }],
            'description': '',
            'subject': {},
            'author': {},
            'goal': [],
            'activity': [{
                'progress': [],
                'detail': {},
            }],
            'note': [],
        }

        if care_plan_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_intent(self, intent: str):
        self.payload_template['intent'] = intent

    def set_category_coding(self, coding: list):
        self.payload_template['category'][0]['coding'] = coding

    def set_description(self, description: str):
        self.payload_template['description'] = description

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_author(self, author: dict):
        self.payload_template['author'] = author

    def set_goal(self, goal: list):
        self.payload_template['goal'] = goal

    def set_activity_progress(self, progress: list):
        self.payload_template['activity'][0]['progress'] = progress

    def set_activity_detail(self, detail: dict):
        self.payload_template['activity'][0]['detail'] = detail

    def set_note(self, note: list):
        self.payload_template['note'] = note

    def create(self):
        return self.payload_template

    def build_care_plan_id_query(self, care_plan_id: str):
        return urlencode({'_id': care_plan_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Coverage:
    def __init__(self, coverage_id=''):
        self.coverage_id = coverage_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Coverage',
            'id': coverage_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'beneficiary': {},
            'payor': [],
        }

        if coverage_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_beneficiary(self, beneficiary: dict):
        self.payload_template['beneficiary'] = beneficiary

    def set_payor(self, payor: list):
        self.payload_template['payor'] = payor

    def create(self):
        return self.payload_template

    def build_coverage_id_query(self, coverage_id: str):
        return urlencode({'_id': coverage_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from fhir_data_generator.patient.patient import Patient
from fhir_data_generator.physical_activity.goal import Goal
from fhir_data_generator.observation.observation import Observation
from fhir_data_generator.physical_activity.care_plan import CarePlan
from fhir_data_generator.physical_activity.patient_ex import PatientEX
from fhir_data_generator.physical_activity.patient_tw import PatientTW
from fhir_data_generator.physical_activity.condition_e import ConditionE
from fhir_data_generator.physical_activity.organization_h import OrganizationH
from fhir_data_generator.physical_activity.organization_s import OrganizationS
from fhir_data_generator.physical_activity.service_request import PhysicalActivityServiceRequest
from fhir_data_generator.physical_activity.observation import Observation as PhysicalActivityObservation
from fhir_data_generator.physical_activity.practitioner import Practitioner as PhysicalActivityPractitioner


from fhir_data_generator.tw_core_ig.media.media import Media as TWCoreMedia
from fhir_data_generator.tw_core_ig.patient.patient import Patient as TWCorePatient
from fhir_data_generator.tw_core_ig.location.location import Location as TWCoreLocation
from fhir_data_generator.tw_core_ig.specimen.specimen import Specimen as TWCoreSpecimen
from fhir_data_generator.tw_core_ig.encounter.encounter import Encounter as TWCoreEncounter
from fhir_data_generator.tw_core_ig.condition.condition import Condition as TWCoreCondition
from fhir_data_generator.tw_core_ig.procedure.procedure import Procedure as TWCoreProcedure
from fhir_data_generator.tw_core_ig.medication.medication import Medication as TWCoreMedication
from fhir_data_generator.tw_core_ig.observation.observation import Observation as TWCoreObservation
from fhir_data_generator.tw_core_ig.composition.composition import Composition as TWCoreComposition
from fhir_data_generator.tw_core_ig.organization.organization import Organization as TWCoreOrganization
from fhir_data_generator.tw_core_ig.practitioner.practitioner import Practitioner as TWCorePractitioner
from fhir_data_generator.tw_core_ig.imaging_study.imaging_study import ImagingStudy as TWCoreImagingStudy
from fhir_data_generator.tw_core_ig.observation.observation_blood_pressure import Observation as TWCoreObservationBloodPressure
from fhir_data_generator.tw_core_ig.observation.observation_lab_report import Observation as TWCoreObservationLabReport
from fhir_data_generator.tw_core_ig.practitioner_role.practitioner_role import PractitionerRole as TWCorePractitionerRole
from fhir_data_generator.tw_core_ig.diagnostic_report.diagnostic_report import DiagnosticReport as TWCoreDiagnosticReport
from fhir_data_generator.tw_core_ig.medication_request.medication_request import MedicationRequest as TWCoreMedicationRequest
from fhir_data_generator.tw_core_ig.document_reference.document_reference import DocumentReference as TWCoreDocumentReference
from fhir_data_generator.tw_core_ig.medication_dispense.medication_dispense import MedicationDispense as TWCoreMedicationDispense
from fhir_data_generator.tw_core_ig.medication_statement.medication_statement import MedicationStatement as TWCoreMedicationStatement
from fhir_data_generator.tw_core_ig.allergyintolerance.allergyintolerance import AllergyIntoleranceNut as TWCoreAllergyIntoleranceNut

from fhir_data_generator.iclaim.patient import Patient as PatientIclaimC1
from fhir_data_generator.iclaim.observation import Observation as ObservationIclaimC1
from fhir_data_generator.iclaim.coverage import Coverage as CoverageC1
from fhir_data_generator.iclaim.claim import Claim

from fhir_data_generator.imri.location import Location as LocationImri
from fhir_data_generator.imri.care_plan import CarePlan as CarePlanImri
from fhir_data_generator.imri.condition import Condition as ConditionImri
from fhir_data_generator.imri.encounter import Encounter as EncounterImri
from fhir_data_generator.imri.composition import Composition as CompositionImri
from fhir_data_generator.imri.imaging_study import ImagingStudy as ImagingStudyImri
from fhir_data_generator.imri.document_reference import DocumentReference as DocumentReferenceImri

from fhir_data_generator.simple_client.simple_client import SimpleClient
from fhir_data_generator.simple_client.keycloak_verify_token import VerifyToken
from fhir_data_generator.simple_client.keycloak_client_credentials import ClientCredentials
from fhir_data_generator.simple_client.keycloak_authorization_code import AuthorizationCode


__version__ = '1.7.0'
__all__ = [
    'Http',
    'CarePlan',
    'Goal',
    'Patient',
    'Observation',
    'ConditionE',
    'PatientEX',
    'PatientTW',
    'OrganizationH',
    'OrganizationS',
    'PhysicalActivityObservation',
    'PhysicalActivityServiceRequest',
    'PhysicalActivityPractitioner',

    'TWCoreAllergyIntoleranceNut',
    'TWCoreComposition',
    'TWCoreCondition',
    'TWCoreDiagnosticReport',
    'TWCoreDocumentReference',
    'TWCoreEncounter',
    'TWCoreImagingStudy',
    'TWCoreLocation',
    'TWCoreMedia',
    'TWCoreMedication',
    'TWCoreMedicationRequest',
    'TWCoreMedicationDispense',
    'TWCoreMedicationStatement',
    'TWCoreObservation',
    'TWCoreOrganization',
    'TWCoreObservationLabReport',
    'TWCoreObservationBloodPressure',
    'TWCorePatient',
    'TWCorePractitioner',
    'TWCorePractitionerRole',
    'TWCoreProcedure',
    'TWCoreSpecimen',

    'PatientIclaimC1',
    'ObservationIclaimC1',
    'CoverageC1',
    'Claim',

    'LocationImri',
    'CarePlanImri',
    'ConditionImri',
    'EncounterImri',
    'CompositionImri',
    'ImagingStudyImri',
    'DocumentReferenceImri',

    'SimpleClient',
    'VerifyToken',
    'ClientCredentials',
    'AuthorizationCode',
]



from urllib.parse import urlencode


class Location:
    def __init__(self, location_id=''):
        self.location_id = location_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Location',
            'id': location_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'name': '',
            'description': '',
            'mode': '',
            'type': [{
                'coding': [],
            }],
            'telecom': [],
            'address': {},
            'position': {},
            'managingOrganization': {},
            'hoursOfOperation': [{
                'daysOfWeek': [],
                'allDay': None,
            }],
        }

        if location_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_name(self, name: str):
        self.payload_template['name'] = name

    def set_description(self, description: str):
        self.payload_template['description'] = description

    def set_mode(self, mode: str):
        self.payload_template['mode'] = mode

    def set_type_coding(self, type_coding: list):
        self.payload_template['type'][0]['coding'] = type_coding

    def set_telecom(self, telecom: list):
        self.payload_template['telecom'] = telecom

    def set_address(self, address: dict):
        self.payload_template['address'] = address

    def set_position(self, position: dict):
        self.payload_template['position'] = position

    def set_managing_organization(self, managing_organization: dict):
        self.payload_template['managingOrganization'] = managing_organization

    def set_hours_of_operation_days_of_week(self, days_of_week: list):
        self.payload_template['hoursOfOperation'][0]['daysOfWeek'] = days_of_week

    def set_hours_of_operation_all_day(self, all_day: bool):
        self.payload_template['hoursOfOperation'][0]['allDay'] = all_day

    def create(self):
        return self.payload_template

    def build_location_id_query(self, location_id: str):
        return urlencode({'_id': location_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Observation:
    def __init__(self, observation_id=''):
        self.observation_id = observation_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Observation',
            'id': observation_id,
            'meta': {
                'profile': [],
            },
            'partOf': [],
            'identifier': [],
            'status': '',
            'category': [{
                'coding': [],
                'text': '',
            }],
            'code': {
                'coding': [],
            },
            'subject': {},
            'issued': '',
            'effectiveDateTime': '',
            'performer': [],
            'valueCodeableConcept': {},
            'valueQuantity': {},
            'interpretation': [],
            'note': [],
            'bodySite': {
                'coding': [],
            },
            'referenceRange': [],
            'valueString': '',
        }

        if observation_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_part_of(self, part_of: list):
        self.payload_template['partOf'] = part_of

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_category_coding(self, category_coding: list):
        self.payload_template['category'][0]['coding'] = category_coding

    def set_category_text(self, category_text: str):
        self.payload_template['category'][0]['text'] = category_text

    def set_code_coding(self, code_coding: list):
        self.payload_template['code']['coding'] = code_coding

    def set_code_text(self, code_text: str):
        self.payload_template['code']['text'] = code_text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_issued(self, issued: str):
        self.payload_template['issued'] = issued

    def set_effective_datetime(self, effective_datetime: str):
        self.payload_template['effectiveDateTime'] = effective_datetime

    def set_performer(self, performer: list):
        self.payload_template['performer'] = performer

    def set_value_codeable_concept(self, value_codeable_concept: dict):
        self.payload_template['valueCodeableConcept'] = value_codeable_concept

    def set_value_quantity(self, value_quantity: dict):
        self.payload_template['valueQuantity'] = value_quantity

    def set_interpretation(self, interpretation: list):
        self.payload_template['interpretation'] = interpretation

    def set_note(self, note: list):
        self.payload_template['note'] = note

    def set_body_site_coding(self, body_site_coding: list):
        self.payload_template['bodySite']['coding'] = body_site_coding

    def set_reference_range(self, reference_range: list):
        self.payload_template['referenceRange'] = reference_range

    def set_value_string(self, value_string: str):
        self.payload_template['valueString'] = value_string

    def create(self):
        if self.payload_template['partOf'] == []:
            del self.payload_template['partOf']
        if self.payload_template['identifier'] == []:
            del self.payload_template['identifier']
        if self.payload_template['issued'] == '':
            del self.payload_template['issued']
        if self.payload_template['valueQuantity'] == {}:
            del self.payload_template['valueQuantity']
        if self.payload_template['interpretation'] == []:
            del self.payload_template['interpretation']
        if self.payload_template['note'] == []:
            del self.payload_template['note']
        if self.payload_template['bodySite'] == {'coding': []}:
            del self.payload_template['bodySite']
        if self.payload_template['referenceRange'] == []:
            del self.payload_template['referenceRange']
        if self.payload_template['valueCodeableConcept'] == {}:
            del self.payload_template['valueCodeableConcept']
        if self.payload_template['valueString'] == '':
            del self.payload_template['valueString']
        if self.payload_template['performer'] == []:
            del self.payload_template['performer']
        if self.payload_template['category'][0]['text'] == '':
            del self.payload_template['category'][0]['text']
        if self.payload_template['category'] == [{'coding': []}]:
            del self.payload_template['category']
        if self.payload_template['code']['coding'] == []:
            del self.payload_template['code']['coding']

        return self.payload_template

    def build_observation_id_query(self, observation_id: str):
        return urlencode({'_id': observation_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Patient:
    def __init__(self, patient_id=''):
        self.patient_id = patient_id

        self.payload_template = {
            'resourceType': 'Patient',
            'id': patient_id,
            'meta': {
                'profile': [],
            },
            'extension': [
                {
                    'url': '',
                    'valueAge' : {},
                },
                {
                    'extension': [{
                        'url': 'code',
                        'valueCodeableConcept': {
                            'coding': [],
                        },
                    }],
                    'url': '',
                },
            ],
            'identifier': [],
            'active': True,
            'name': [{
                'use': '',
                'text': '',
                'family': '',
                'given': [],
            }],
            'telecom': [],
            'gender': '',
            'birthDate': '',
            'address': [{
                'extension': [],
                'text': '',
                'line': [],
                'city': '',
                'district': '',
                '_postalCode': {
                    'extension': [],
                },
                'country': 'TW',
            }],
            'maritalStatus': {
                'coding': [],
            },
            'photo': [],
            'contact': [{
                'relationship': [{
                    'coding': [],
                }],
                'name': {},
                'telecom': [],
            }],
            'communication': [{
                'language': {
                    'coding': [],
                },
            }],
            'managingOrganization': {},
        }

        if patient_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_extension_url(self, extension_url: str):
        self.payload_template['extension'][0]['url'] = extension_url

    def set_extension_value_age(self, value_age: dict):
        self.payload_template['extension'][0]['valueAge'] = value_age

    def set_extension_extension_coding(self, extension_coding: list):
        self.payload_template['extension'][1]['extension'][0]['valueCodeableConcept']['coding'] = extension_coding

    def set_extension_extension_url(self, extension_extension_url: str):
        self.payload_template['extension'][1]['url'] = extension_extension_url

    def set_identifiers(self, identifiers: list):
        self.payload_template['identifier'] = identifiers

    def set_active(self, active: bool):
        self.payload_template['active'] = active

    def set_name_use(self, name_use: str):
        self.payload_template['name'][0]['use'] = name_use

    def set_name_text(self, name_text: str):
        self.payload_template['name'][0]['text'] = name_text

    def set_name_family(self, name_family: str):
        self.payload_template['name'][0]['family'] = name_family

    def set_name_given(self, name_given: str):
        self.payload_template['name'][0]['given'] = name_given

    def set_telecom(self, telecom: list):
        self.payload_template['telecom'] = telecom

    def set_gender(self, gender: str):
        self.payload_template['gender'] = gender

    def set_birth_date(self, birth_date: str):
        self.payload_template['birthDate'] = birth_date

    def set_address_extension(self, address_extension: list):
        self.payload_template['address'][0]['extension'] = address_extension

    def set_address_text(self, address_text: str):
        self.payload_template['address'][0]['text'] = address_text

    def set_address_line(self, address_line: list):
        self.payload_template['address'][0]['line'] = address_line

    def set_address_city(self, address_city: str):
        self.payload_template['address'][0]['city'] = address_city

    def set_address_district(self, address_district: str):
        self.payload_template['address'][0]['district'] = address_district

    def set_address_postal_code_extension(self, address_postal_code: list):
        self.payload_template['address'][0]['_postalCode']['extension'] = address_postal_code

    def set_marital_status_coding(self, marital_status_coding: list):
        self.payload_template['maritalStatus']['coding'] = marital_status_coding

    def set_photo(self, photo: list):
        self.payload_template['photo'] = photo

    def set_contact(self, contact_relationship_coding: dict):
        self.payload_template['contact'][0]['relationship'][0]['coding'] = contact_relationship_coding

    def set_contact_name(self, contact_name: dict):
        self.payload_template['contact'][0]['name'] = contact_name

    def set_contact_telecom(self, contact_telecom: list):
        self.payload_template['contact'][0]['telecom'] = contact_telecom

    def set_communication_language_coding(self, communication_lang_coding: list):
        self.payload_template['communication'][0]['language']['coding'] = communication_lang_coding

    def set_managing_organization(self, managing_organization: dict):
        self.payload_template['managingOrganization'] = managing_organization

    def create(self):
        return self.payload_template

    def build_patient_id_query(self, patient_id: str):
        return urlencode({'_id': patient_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class AllergyIntoleranceNut:
    def __init__(self, allergy_intolerance_id=''):
        self.allergy_intolerance_id = allergy_intolerance_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'AllergyIntolerance',
            'id': allergy_intolerance_id,
            'meta': {
                'profile': [],
            },
            'clinicalStatus': {
                'coding': [],
                'text': '',
            },
            'verificationStatus': {
                'coding': [],
                'text': '',
            },
            'type': 'allergy',
            'category': [],
            'criticality': '',
            'code': {
                'coding': [],
                'text': '',
            },
            'patient': {},
            'onsetDateTime': '',
            'recordedDate': '',
            'recorder': {},
            'asserter': {},
            'lastOccurrence': '',
            'note': [],
            'reaction' : [{
                'substance' : {
                    'coding' : [{
                        'system' : 'http://snomed.info/sct',
                        'code' : '3193000',
                        'display' : 'alpha-1,4-Glucan-protein synthase (uridine diphosphate-forming)'
                    }],
                    'text' : 'alpha-1,4-Glucan-protein synthase (uridine diphosphate-forming)'
                },
                'manifestation' : [{
                    'coding' : [{
                        'system' : 'http://snomed.info/sct',
                        'code' : '39579001',
                        'display' : 'Anaphylaxis (disorder)'
                    }]
                }],
                'description' : '',
                'severity' : '',
                'exposureRoute' : {
                    'coding' : [{
                        'system' : 'http://snomed.info/sct',
                        'code' : '26643006',
                        'display' : 'Oral use'
                    }],
                    'text' : ''
                },
                'note' : [],
            }]
        }

        if allergy_intolerance_id == '':
            del self.payload_template['id']

    def set_allergy_intolerance_id(self, allergy_intolerance_id):
        self.allergy_intolerance_id = allergy_intolerance_id

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_clinical_status_coding(self, coding: list):
        self.payload_template['clinicalStatus']['coding'] = coding

    def set_clinical_status_text(self, text: str):
        self.payload_template['clinicalStatus']['text'] = text

    def set_verification_status_coding(self, coding: list):
        self.payload_template['verificationStatus']['coding'] = coding

    def set_verification_status_text(self, text: str):
        self.payload_template['verificationStatus']['text'] = text

    def set_category(self, category: list):
        self.payload_template['category'] = category

    def set_criticality(self, criticality: str):
        self.payload_template['criticality'] = criticality

    def set_code_coding(self, coding: list):
        self.payload_template['code']['coding'] = coding

    def set_code_text(self, text: str):
        self.payload_template['code']['text'] = text

    def set_patient(self, patient: dict):
        self.payload_template['patient'] = patient

    def set_onset_datetime(self, onset_datetime: str):
        self.payload_template['onsetDateTime'] = onset_datetime

    def set_recorded_date(self, recorded_date: str):
        self.payload_template['recordedDate'] = recorded_date

    def set_recorder(self, recorder: dict):
        self.payload_template['recorder'] = recorder

    def set_asserter(self, asserter: dict):
        self.payload_template['asserter'] = asserter

    def set_last_occurrence(self, last_occurrence: str):
        self.payload_template['lastOccurrence'] = last_occurrence

    def set_note(self, note: list):
        self.payload_template['note'] = note

    def set_reaction_description(self, description: str):
        self.payload_template['reaction'][0]['description'] = description

    def set_reaction_severity(self, severity: str):
        self.payload_template['reaction'][0]['severity'] = severity

    def set_reaction_exposure_route_text(self, text: str):
        self.payload_template['reaction'][0]['exposureRoute']['text'] = text

    def set_reaction_note(self, reaction_note: list):
        self.payload_template['reaction'][0]['note'] = reaction_note

    def create(self):
        return self.payload_template

    def build_allergy_intolerance_id_query(self, allergy_intolerance_id: str):
        return urlencode({'_id': allergy_intolerance_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Specimen:
    def __init__(self, specimen_id=''):
        self.specimen_id = specimen_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Specimen',
            'id': specimen_id,
            'meta': {
                'profile': [],
            },
            'identifier': [],
            'accessionIdentifier': {},
            'status': '',
            'type': {
                'coding': [],
            },
            'subject': {},
            'receivedTime': '',
            'collection': {
                'collector': {},
                'collectedDateTime': '',
                'quantity': {},
                'method': {
                    'coding': [],
                    'text': '',
                },
                'bodySite': {
                    'coding': [],
                    'text': '',
                },
                'fastingStatusCodeableConcept': {
                    'coding': [],
                },
            },
            'processing': [],
            'container': [],
            'note': [],
        }

        if specimen_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_accession_identifier(self, accession_identifier: dict):
        self.payload_template['accessionIdentifier'] = accession_identifier

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_type_coding(self, type_coding: list):
        self.payload_template['type']['coding'] = type_coding

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_received_time(self, received_time: str):
        self.payload_template['receivedTime'] = received_time

    def set_collection_collector(self, collection_collector: dict):
        self.payload_template['collection']['collector'] = collection_collector

    def set_collection_collected_date_time(self, collection_collected_date_time: str):
        self.payload_template['collection']['collectedDateTime'] = collection_collected_date_time

    def set_collection_quantity(self, collection_quantity: dict):
        self.payload_template['collection']['quantity'] = collection_quantity

    def set_collection_method_coding(self, collection_method_coding: list):
        self.payload_template['collection']['method']['coding'] = collection_method_coding

    def set_collection_method_text(self, collection_method_text: str):
        self.payload_template['collection']['method']['text'] = collection_method_text

    def set_collection_body_site_coding(self, collection_body_site_coding: list):
        self.payload_template['collection']['bodySite']['coding'] = collection_body_site_coding

    def set_collection_body_site_text(self, collection_body_site_text: str):
        self.payload_template['collection']['bodySite']['text'] = collection_body_site_text

    def set_collection_fasting_status_codeable_concept_coding(self, coding: list):
        self.payload_template['collection']['fastingStatusCodeableConcept']['coding'] = coding

    def set_processing(self, processing: list):
        self.payload_template['processing'] = processing

    def set_container(self, container: list):
        self.payload_template['container'] = container

    def set_note(self, note: list):
        self.payload_template['note'] = note

    def create(self):
        return self.payload_template

    def build_specimen_id_query(self, specimen_id: str):
        return urlencode({'_id': specimen_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Practitioner:
    def __init__(self, practitioner_id=''):
        self.practitioner_id = practitioner_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Practitioner',
            'id': practitioner_id,
            'meta': {
                'profile': [],
            },
            'identifier' : [],
            'active': True,
            'name': [{
                'text': '',
            }],
        }

        if practitioner_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifiers(self, identifiers: list):
        self.payload_template['identifier'] = identifiers

    def set_active(self, active: bool):
        self.payload_template['active'] = active

    def set_name_text(self, name_text: str):
        self.payload_template['name'][0]['text'] = name_text

    def create(self):
        return self.payload_template

    def build_practitioner_id_query(self, practitioner_id: str):
        return urlencode({'_id': practitioner_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



import json
import requests


class SimpleClient:
    def __init__(self, server_url):
        self.server_url = server_url
        self.headers = {
            'Accept': 'application/fhir+json',
            'Content-Type': 'application/fhir+json',
        }

    def send(self, path, http_method, headers, json_payload=None):
        for value in enumerate(headers):
            self.headers[value[1]] = headers[value[1]]

        headers = self.headers
        req_url = f'{self.server_url}{path}'

        requester = getattr(requests, http_method)

        if json_payload is None:
            response = requester(req_url, headers=headers)
        else:
            json_str = json.dumps(json_payload)
            response = requester(req_url, headers=headers, data=json_str)

        return response

    def handle_response(self, response):
        if hasattr(response, 'status_code') is False:
            raise AttributeError(f'Given response has invalid status_code attribute!')
        if hasattr(response, 'text') is False:
            raise AttributeError(f'Given response has invalid text attribute!')
        if hasattr(response, 'json') is False:
            raise AttributeError(f'Given response has invalid json attribute!')
        if callable(response.json) is False:
            raise TypeError(f'Given response has invalid json attribute!')

        handled_response = {
            'status': response.status_code,
            'text': response.text,
            'json': response.json(),
        }

        return handled_response



from urllib.parse import urlencode


class ImagingStudy:
    def __init__(self, imaging_study_id=''):
        self.imaging_study_id = imaging_study_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'ImagingStudy',
            'id': imaging_study_id,
            'meta': {
                'profile': [],
            },
            'identifier': [],
            'status': '',
            'subject': {},
            'encounter': {},
            'started': '',
            'numberOfSeries': None,
            'numberOfInstances': None,
            'procedureReference': {},
            'procedureCode': [{
                'coding': [],
            }],
            'series': [{
                'uid': '',
                'modality': {},
                'bodySite': {},
                'performer': [{
                    'actor': {},
                }],
                'instance': [{
                    'uid': '',
                    'sopClass': {},
                }],
            }],
        }

        if imaging_study_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_encounter(self, encounter: dict):
        self.payload_template['encounter'] = encounter

    def set_started(self, started: str):
        self.payload_template['started'] = started

    def set_number_of_series(self, number_of_series: int):
        self.payload_template['numberOfSeries'] = number_of_series

    def set_number_of_instances(self, number_of_instances: int):
        self.payload_template['numberOfInstances'] = number_of_instances

    def set_procedure_reference(self, procedure_reference: dict):
        self.payload_template['procedureReference'] = procedure_reference

    def set_procedure_code_coding(self, procedure_code_coding: list):
        self.payload_template['procedureCode'][0]['coding'] = procedure_code_coding

    def set_series_uid(self, uid: str):
        self.payload_template['series'][0]['uid'] = uid

    def set_series_modality(self, modality: dict):
        self.payload_template['series'][0]['modality'] = modality

    def set_series_body_site(self, body_site: dict):
        self.payload_template['series'][0]['bodySite'] = body_site

    def set_series_performer_actor(self, performer_actor: dict):
        self.payload_template['series'][0]['performer'][0]['actor'] = performer_actor

    def set_series_instance_uid(self, uid: str):
        self.payload_template['series'][0]['instance'][0]['uid'] = uid

    def set_series_instance_sop_class(self, sop_class: dict):
        self.payload_template['series'][0]['instance'][0]['sopClass'] = sop_class

    def create(self):
        return self.payload_template

    def build_imaging_study_id_query(self, imaging_study_id: str):
        return urlencode({'_id': imaging_study_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class MedicationRequest:
    def __init__(self, medication_request_id=''):
        self.medication_request_id = medication_request_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'MedicationRequest',
            'id': medication_request_id,
            'meta': {
                'profile': [],
            },
            'identifier': [],
            'status': '',
            'statusReason': {
                'coding': [],
            },
            'intent': '',
            'medicationCodeableConcept': {},
            'category': [{
                'coding': [],
            }],
            'medicationReference': {},
            'subject': {},
            'encounter': {},
            'authoredOn': {},
            'requester': {},
            'performer': {},
            'basedOn': [],
            'reasonReference': [],
            'dosageInstruction': [{
                'text': '',
                'timing': {
                    'code': {
                        'coding': [],
                        'text': '',
                    },
                },
                'route': {
                    'coding': [],
                },
                'doseAndRate': [{
                    'type': {
                        'coding': [],
                    },
                }],
            }],
            'dispenseRequest': {
                'validityPeriod': {
                    'start': '',
                    'end': '',
                },
            },
        }

        if medication_request_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_status_reason_coding(self, status_reason_coding: list):
        self.payload_template['statusReason']['coding'] = status_reason_coding

    def set_intent(self, intent: str):
        self.payload_template['intent'] = intent

    def set_medication_codeable_concept(self, medication_codeable_concept: dict):
        self.payload_template['medicationCodeableConcept'] = medication_codeable_concept

    def set_category_coding(self, category_coding: list):
        self.payload_template['category'][0]['coding'] = category_coding

    def set_medication_reference(self, medication_reference: dict):
        self.payload_template['medicationReference'] = medication_reference

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_encounter(self, encounter: dict):
        self.payload_template['encounter'] = encounter

    def set_authored_on(self, authored_on: dict):
        self.payload_template['authoredOn'] = authored_on

    def set_requester(self, requester: dict):
        self.payload_template['requester'] = requester

    def set_performer(self, performer: dict):
        self.payload_template['performer'] = performer

    def set_based_on(self, based_on: list):
        self.payload_template['basedOn'] = based_on

    def set_reason_reference(self, reason_reference: list):
        self.payload_template['reasonReference'] = reason_reference

    def set_dosage_instruction_text(self, dosage_instruction_text: str):
        self.payload_template['dosageInstruction'][0]['text'] = dosage_instruction_text

    def set_dosage_instruction_timing_coding(self, dosage_instruction_timing_coding: list):
        self.payload_template['dosageInstruction'][0]['timing']['code']['coding'] = dosage_instruction_timing_coding

    def set_dosage_instruction_timing_text(self, dosage_instruction_timing_text: str):
        self.payload_template['dosageInstruction'][0]['timing']['code']['text'] = dosage_instruction_timing_text

    def set_dosage_instruction_route_coding(self, route_coding: list):
        self.payload_template['dosageInstruction'][0]['route']['coding'] = route_coding

    def set_dosage_instruction_dose_rate_type_coding(self, type_coding: list):
        self.payload_template['dosageInstruction'][0]['doseAndRate'][0]['type']['coding'] = type_coding

    def set_dispense_request_validity_period_start(self, validity_period_start: str):
        self.payload_template['dispenseRequest']['validityPeriod']['start'] = validity_period_start

    def set_dispense_request_validity_period_end(self, validity_period_end: str):
        self.payload_template['dispenseRequest']['validityPeriod']['end'] = validity_period_end

    def create(self):
        if self.payload_template['identifier'] == []:
            del self.payload_template['identifier']
        if self.payload_template['statusReason'] == {'coding': []}:
            del self.payload_template['statusReason']
        if self.payload_template['category'] == [{'coding': []}]:
            del self.payload_template['category']
        if self.payload_template['medicationCodeableConcept'] == {}:
            del self.payload_template['medicationCodeableConcept']
        if self.payload_template['medicationReference'] == {}:
            del self.payload_template['medicationReference']
        if self.payload_template['authoredOn'] == {}:
            del self.payload_template['authoredOn']
        if self.payload_template['reasonReference'] == []:
            del self.payload_template['reasonReference']
        if self.payload_template['performer'] == {}:
            del self.payload_template['performer']
        if self.payload_template['basedOn'] == []:
            del self.payload_template['basedOn']
        if self.payload_template['dosageInstruction'] == [{
                'text': '',
                'timing': {
                    'code': {
                        'coding': [],
                        'text': '',
                    },
                },
                'route': {
                    'coding': [],
                },
                'doseAndRate': [{
                    'type': {
                        'coding': [],
                    },
                }],
            }]:
            del self.payload_template['dosageInstruction']
        if self.payload_template['dispenseRequest'] == {'validityPeriod': {'start': '', 'end': ''}}:
            del self.payload_template['dispenseRequest']

        return self.payload_template

    def build_medication_request_id_query(self, medication_request_id: str):
        return urlencode({'_id': medication_request_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



import time
import requests
from bs4 import BeautifulSoup
from urllib.parse import urlsplit, parse_qs, urlencode


class AuthorizationCode:
    def __init__(self, client_id: str, client_secret: str, redirect_uri: str, req_auth_code_url: str, access_token_req_url: str, form_action_payload: dict):
        self.client_id = client_id
        self.client_secret = client_secret
        self.redirect_uri = redirect_uri
        self.grant_type = 'authorization_code'
        self.scope = 'openid'
        self.response_type = 'code'
        self.req_auth_code_url = req_auth_code_url
        self.access_token_req_url = access_token_req_url
        self.authorization_code = None
        self.req_session = None
        self.response = None
        self.req_auth_code_headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
        }
        self.auth_code_headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Accept': 'application/json',
        }
        self.form_action_payload = form_action_payload

        if form_action_payload.get('username') is None:
            raise Exception('username is not found in the form_action_payload!')
        if form_action_payload.get('password') is None:
            raise Exception('password is not found in the form_action_payload!')

    def retrieve_authorization_code(self):
        req_session = requests.Session()

        encoded_query = urlencode(
            {
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'scope': self.scope,
                'response_type': self.response_type,
                'redirect_uri': self.redirect_uri,
            }
        )

        req_auth_code_url = f'{self.req_auth_code_url}?{encoded_query}'
        response = req_session.get(req_auth_code_url)

        if response.ok is False:
            return response.json()

        time.sleep(1)
        soup = BeautifulSoup(response.text, 'html.parser')
        form_action = soup.select_one('form')

        if form_action.get('action') is None:
            raise Exception('Retrieving form action URL is failed')

        form_action_url = form_action['action']
        response = req_session.post(form_action_url, headers=self.req_auth_code_headers, data=self.form_action_payload, allow_redirects=False)

        if response.headers.get('Location') is None:
            raise Exception('Location header is not found')

        parsed_url = urlsplit(response.headers['Location'])
        parsed_query = parse_qs(parsed_url.query)

        if parsed_query.get('code') is None or len(parsed_query['code']) < 1:
            raise Exception('code param is not found')

        authorization_code = parsed_query['code'][0]
        self.authorization_code = authorization_code
        self.req_session = req_session

        return authorization_code

    def send(self):
        payload = urlencode(
            {
                'client_id': self.client_id,
                'client_secret': self.client_secret,
                'code': self.authorization_code,
                'redirect_uri': self.redirect_uri,
                'grant_type': self.grant_type,
            }
        )

        response = self.req_session.post(self.access_token_req_url, headers=self.auth_code_headers, data=payload, allow_redirects=False)
        self.response = response

        return response

    def retrieve_token(self):
        if self.response.status_code != 200:
            raise Exception(f'The response status code is {self.response.status_code}')

        return self.response.json().get('access_token')



from urllib.parse import urlencode


class Encounter:
    def __init__(self, encounter_id=''):
        self.encounter_id = encounter_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Encounter',
            'id': encounter_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'class': {},
            'serviceType': {
                'coding': [],
            },
            'subject': {},
            'participant': [{
                'individual': {},
            }],
            'period': {},
            'hospitalization': {},
        }

        if encounter_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_class(self, the_class: dict):
        self.payload_template['class'] = the_class

    def set_service_type_coding(self, service_type_coding: list):
        self.payload_template['serviceType']['coding'] = service_type_coding

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_participant_individual(self, participant_individual: dict):
        self.payload_template['participant'][0]['individual'] = participant_individual

    def set_period(self, period: dict):
        self.payload_template['period'] = period

    def set_hospitalization(self, hospitalization: dict):
        self.payload_template['hospitalization'] = hospitalization

    def create(self):
        return self.payload_template

    def build_encounter_id_query(self, encounter_id: str):
        return urlencode({'_id': encounter_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class PhysicalActivityServiceRequest:
    def __init__(self, service_request_id=''):
        self.service_request_id = service_request_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'ServiceRequest',
            'id': service_request_id,
            'meta': {
                'profile': [],
            },
            'identifier': [],
            'status': '',
            'intent': '',
            'category': [{
                'coding': [],
            }],
            'code': {
                'coding': [],
            },
            'subject': {},
            'authoredOn': '',
            'requester': {},
        }

        if service_request_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifiers(self, identifiers: list):
        self.payload_template['identifier'] = identifiers

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_intent(self, intent: str):
        self.payload_template['intent'] = intent

    def set_category_coding(self, category_coding: list):
        self.payload_template['category'][0]['coding'] = category_coding

    def set_code_coding(self, code_coding: list):
        self.payload_template['code']['coding'] = code_coding

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_authored_on(self, authored_on: str):
        self.payload_template['authoredOn'] = authored_on

    def set_requester(self, requester: dict):
        self.payload_template['requester'] = requester

    def create(self):
        return self.payload_template

    def build_service_request_id_query(self, service_request_id: str):
        return urlencode({'_id': service_request_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Location:
    def __init__(self, location_id=''):
        self.location_id = location_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Location',
            'id': location_id,
            'meta': {
                'profile': [],
            },
            'identifier': [],
            'name': '',
        }

        if location_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_name(self, name: str):
        self.payload_template['name'] = name

    def create(self):
        return self.payload_template

    def build_location_id_query(self, location_id: str):
        return urlencode({'_id': location_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class MedicationDispense:
    def __init__(self, medication_dispense_id=''):
        self.medication_dispense_id = medication_dispense_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'MedicationDispense',
            'id': medication_dispense_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'category': {
                'coding': [],
            },
            'medicationReference': {},
            'subject': {},
            'context': {},
            'performer': [{
                'actor': {},
            }],
            'type': {
                'coding': [],
            },
            'quantity': {},
            'daysSupply': {},
            'whenPrepared': '',
            'whenHandedOver': '',
            'dosageInstruction': [],
            'substitution': {
                'wasSubstituted': None,
                'type': {
                    'coding': [],
                },
                'reason': [{
                    'coding': [],
                }],
            },
        }

        if medication_dispense_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_category_coding(self, category_coding: list):
        self.payload_template['category']['coding'] = category_coding

    def set_medication_reference(self, medication_reference: dict):
        self.payload_template['medicationReference'] = medication_reference

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_context(self, context: dict):
        self.payload_template['context'] = context

    def set_performer_actor(self, performer_actor: dict):
        self.payload_template['performer'][0]['actor'] = performer_actor

    def set_type_coding(self, type_coding: dict):
        self.payload_template['type']['coding'] = type_coding

    def set_quantity(self, quantity: dict):
        self.payload_template['quantity'] = quantity

    def set_days_supply(self, days_supply: dict):
        self.payload_template['daysSupply'] = days_supply

    def set_when_prepared(self, when_prepared: str):
        self.payload_template['whenPrepared'] = when_prepared

    def set_when_handed_over(self, when_handed_over: str):
        self.payload_template['whenHandedOver'] = when_handed_over

    def set_dosage_instruction(self, dosage_instruction: list):
        self.payload_template['dosageInstruction'] = dosage_instruction

    def set_substitution_was_substituted(self, was_substituted: bool):
        self.payload_template['substitution']['wasSubstituted'] = was_substituted

    def set_substitution_type_coding(self, substitution_type_coding: list):
        self.payload_template['substitution']['type']['coding'] = substitution_type_coding

    def set_substitution_reason_coding(self, substitution_reason_coding: list):
        self.payload_template['substitution']['reason'][0]['coding'] = substitution_reason_coding

    def create(self):
        return self.payload_template

    def build_medication_dispense_id_query(self, medication_dispense_id: str):
        return urlencode({'_id': medication_dispense_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



import requests


class VerifyToken:
    def __init__(self, req_url: str, client_id: str, access_token: str):
        self.req_url = req_url
        self.headers = {'client_id': client_id, 'Authorization': f'Bearer {access_token}'}

    def is_verified(self):
        response = requests.get(self.req_url, headers=self.headers)

        return response.ok



from urllib.parse import urlencode


class Encounter:
    def __init__(self, encounter_id=''):
        self.encounter_id = encounter_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Encounter',
            'id': encounter_id,
            'meta': {
                'profile': [],
            },
            'extension': [],
            'identifier': [],
            'status': '',
            'class': {},
            'type': [{
                'coding': [],
            }],
            'serviceType': {
                'coding': [],
                'text': '',
            },
            'subject': {},
            'participant': [{
                'type': [{
                    'coding': [],
                }],
                'period': {},
                'individual': {},
            }],
            'period': {},
            'length': {},
            'reasonCode': [{
                'coding': [],
            }],
            'hospitalization': {
                'dischargeDisposition': {
                    'coding': [],
                },
            },
            'location': [{
                'location': {},
            }],
        }

        if encounter_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_extension(self, extension: list):
        self.payload_template['extension'] = extension

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_class(self, the_class: dict):
        self.payload_template['class'] = the_class

    def set_type_coding(self, type_coding: list):
        self.payload_template['type'][0]['coding'] = type_coding

    def set_service_type_coding(self, service_type_coding: list):
        self.payload_template['serviceType']['coding'] = service_type_coding

    def set_service_type_text(self, service_type_text: str):
        self.payload_template['serviceType']['text'] = service_type_text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_participant_type_coding(self, participant_type_coding: list):
        self.payload_template['participant'][0]['type'][0]['coding'] = participant_type_coding

    def set_participant_period(self, participant_period: dict):
        self.payload_template['participant'][0]['period'] = participant_period

    def set_participant_individual(self, participant_individual: dict):
        self.payload_template['participant'][0]['individual'] = participant_individual

    def set_period(self, period: dict):
        self.payload_template['period'] = period

    def set_length(self, length: dict):
        self.payload_template['length'] = length

    def set_reason_code_coding(self, reason_code_coding: list):
        self.payload_template['reasonCode'][0]['coding'] = reason_code_coding

    def set_hospitalization_discharge_disposition_coding(self, coding: list):
        self.payload_template['hospitalization']['dischargeDisposition']['coding'] = coding

    def set_location(self, location_location: dict):
        self.payload_template['location'][0]['location'] = location_location

    def create(self):
        if len(self.payload_template['extension']) == 0:
            del self.payload_template['extension']

        if self.payload_template['reasonCode'] == [{'coding': []}]:
            del self.payload_template['reasonCode']
        if self.payload_template['location'] == [{'location': {}}]:
            del self.payload_template['location']
        if self.payload_template['length'] == {}:
            del self.payload_template['length']
        if self.payload_template['hospitalization'] == {'dischargeDisposition': {'coding': []}}:
            del self.payload_template['hospitalization']
        if self.payload_template['type'] == [{'coding': []}]:
            del self.payload_template['type']
        if self.payload_template['serviceType']['text'] == '':
            del self.payload_template['serviceType']['text']

        if self.payload_template['participant'][0]['period'] == {}:
            del self.payload_template['participant'][0]['period']
        if self.payload_template['participant'][0]['type'] == [{'coding': []}]:
            del self.payload_template['participant'][0]['type']

        return self.payload_template

    def build_encounter_id_query(self, encounter_id: str):
        return urlencode({'_id': encounter_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Composition:
    def __init__(self, composition_id=''):
        self.composition_id = composition_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Composition',
            'id': composition_id,
            'meta': {
                'profile': [],
            },
            'identifier': {},
            'status': '',
            'type': {
                'coding': [],
            },
            'category': [],
            'subject': {},
            'date': '',
            'author': [],
            'title': '',
            'confidentiality': '',
            'attester': [],
            'custodian': {},
            'event': [{
                'code': [],
                'period': {},
            }],
            'section': [{
                'title': '',
                'code': {},
                'entry': [],
            }],
        }

        if composition_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifier(self, identifier: dict):
        self.payload_template['identifier'] = identifier

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_type_coding(self, type_coding: list):
        self.payload_template['type']['coding'] = type_coding

    def set_category(self, category: list):
        self.payload_template['category'] = category

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_date(self, date: str):
        self.payload_template['date'] = date

    def set_author(self, author: list):
        self.payload_template['author'] = author

    def set_title(self, title: str):
        self.payload_template['title'] = title

    def set_confidentiality(self, confidentiality: str):
        self.payload_template['confidentiality'] = confidentiality

    def set_attester(self, attester: list):
        self.payload_template['attester'] = attester

    def set_custodian(self, custodian: dict):
        self.payload_template['custodian'] = custodian

    def set_event_code(self, event_code: list):
        self.payload_template['event'][0]['code'] = event_code

    def set_event_period(self, period: dict):
        self.payload_template['event'][0]['period'] = period

    def set_section_title(self, title: str):
        self.payload_template['section'][0]['title'] = title

    def set_section_code(self, code: dict):
        self.payload_template['section'][0]['code'] = code

    def set_section_entry(self, entry: list):
        self.payload_template['section'][0]['entry'] = entry

    def create(self):
        if self.payload_template['identifier'] == {}:
            del self.payload_template['identifier']
        if self.payload_template['category'] == []:
            del self.payload_template['category']
        if self.payload_template['confidentiality'] == '':
            del self.payload_template['confidentiality']
        if self.payload_template['attester'] == []:
            del self.payload_template['attester']
        if self.payload_template['custodian'] == {}:
            del self.payload_template['custodian']
        if self.payload_template['event'] == [{'code': [], 'period': {}}]:
            del self.payload_template['event']
        if self.payload_template['section'][0]['title'] == '':
            del self.payload_template['section'][0]['title']
        if self.payload_template['section'][0]['code'] == {}:
            del self.payload_template['section'][0]['code']

        return self.payload_template

    def build_composition_id_query(self, composition_id: str):
        return urlencode({'_id': composition_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Observation:
    def __init__(self, observation_id):
        self.observation_id = observation_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Observation',
            'id': observation_id,
            'meta': {
                'profile': [],
            },
            'status': '',
            'category': [{
                'coding': [],
            }],
            'code': {
                'coding': [],
                'text': '',
            },
            'subject': {},
            'effectiveDateTime': '',
            'performer': [],
            'valueQuantity': {},
        }

    def set_observation_id(self, observation_id):
        self.observation_id = observation_id

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_status(self, status: str):
        self.payload_template['status'] = status

    def set_category_coding(self, coding: list):
        self.payload_template['category'][0]['coding'] = coding

    def set_code_coding(self, coding: list):
        self.payload_template['code']['coding'] = coding

    def set_code_text(self, text: str):
        self.payload_template['code']['text'] = text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_effective_datetime(self, effective_datetime: str):
        self.payload_template['effectiveDateTime'] = effective_datetime

    def set_performer(self, performer: list):
        self.payload_template['performer'] = performer

    def set_value_quantity(self, value_quantity: dict):
        self.payload_template['valueQuantity'] = value_quantity

    def create(self):
        return self.payload_template

    def build_observation_id_query(self, observation_id: str):
        return urlencode({'_id': observation_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)



from urllib.parse import urlencode


class Condition:
    def __init__(self, condition_id=''):
        self.condition_id = condition_id

        self.profile_urls = []

        self.payload_template = {
            'resourceType': 'Condition',
            'id': condition_id,
            'meta': {
                'profile': [],
            },
            'clinicalStatus': {
                'coding': [],
                'text': '',
            },
            'category': [{
                'coding': [],
            }],
            'code': {
                'coding': [],
                'text': '',
            },
            'subject': {},
            'encounter': {},
            'note': [],
            'recordedDate': '',
        }

        if condition_id == '':
            del self.payload_template['id']

    def set_profile_urls(self, profile_urls: list):
        self.payload_template['meta']['profile'] = profile_urls

    def set_identifier(self, identifier: list):
        self.payload_template['identifier'] = identifier

    def set_clinical_status_coding(self, clinical_status_coding: list):
        self.payload_template['clinicalStatus']['coding'] = clinical_status_coding

    def set_clinical_status_text(self, clinical_status_text: str):
        self.payload_template['clinicalStatus']['text'] = clinical_status_text

    def set_category_coding(self, category_coding: list):
        self.payload_template['category'][0]['coding'] = category_coding

    def set_code_coding(self, code_coding: list):
        self.payload_template['code']['coding'] = code_coding

    def set_code_text(self, code_text: str):
        self.payload_template['code']['text'] = code_text

    def set_subject(self, subject: dict):
        self.payload_template['subject'] = subject

    def set_encounter(self, encounter: dict):
        self.payload_template['encounter'] = encounter

    def set_recorded_date(self, recorded_date: str):
        self.payload_template['recordedDate'] = recorded_date

    def set_note(self, note: list):
        self.payload_template['note'] = note

    def create(self):
        if self.payload_template['clinicalStatus']['text'] == '':
            del self.payload_template['clinicalStatus']['text']
        if self.payload_template['code']['text'] == '':
            del self.payload_template['code']['text']
        if self.payload_template['recordedDate'] == '':
            del self.payload_template['recordedDate']
        if self.payload_template['encounter'] == {}:
            del self.payload_template['encounter']
        if self.payload_template['note'] == []:
            del self.payload_template['note']

        return self.payload_template

    def build_condition_id_query(self, condition_id: str):
        return urlencode({'_id': condition_id})

    def build_search_param(self, query_params: dict):
        return urlencode(query_params)
